classdef BSolverComposite < BSolver
    % Abstract class for estimating (fitting) base terms from a *Signal*.
    % It relies on the use of an appropriate BSolverSimple and CSolver to
    % obtain the solution.
    %
    % Inherits BSolver.
    %
    methods
        %------------------------------------------------------------------------------------------
        function init(self,varargin)
            % [constructor] Create a base term solver that uses the
            % sub-sampling parametric method. An sub-sampled signal may
            % present better conditioned matrices than the full signal.
            % Downsampling the signal introduces an aliasing problem that
            % can be solved by computing the intersection with a second
            % sub-sampling of the signal that is shifted. Details are
            % found in "HOW DOWNSAMPLING CAN LEAD TO MORE ROBUSTNESS AND
            % HIGHER RESOLUTION IN PARAMETRIC SPECTRAL ANALYSIS" by Lee and
            % Cuyt.
            %
            % This method makes use of appropriate BSolver and CSolver
            % objects to obtain the solution.
            %
            % Additional parameters are:
            %
            % :param key,value,BSolver [bsolver]: BSolver object to use for
            %   solving the sub-sampled signal. Default is a default
            %   constructed BSolverEsprit. 
            % :param key,value,CSolver [csolver]: CSolver object to use for
            %   solving the sub-sampled signal. Default is a default
            %   constructed CSolverVandermondeLS.  
            % :return: a BSolverComposite object.
            %
            % `examples <tutorial_BSolver.html>`_
            %
            
            self.init@BSolver();
            self.parms.add_parameters ...
              ( {'bsolver:BSolver:BSolver for solving the signal' , BSolverEsprit ...
                ,'csolver:CSolver:CSolver for solving the signal' , CSolverVandermondeLS ...
                } ...
              , varargin{:} ...
              );
            
            % further input validation:
            if isa(self.bsolver,class(self)) % avoid inadvertent recursion
                Sparsimatics.Error( fa.get_signature ...
                  , 'Cannot use a BSolverComposite as a Bsolver.'...
                  );
            end
        end
        %------------------------------------------------------------------------------------------
        function s = get_display_struct(self)
            c = [{'BSolver';class(self.bsolver)} destruct(self.bsolver.get_display_struct,'bsolver_') ...
                 {'CSolver';class(self.csolver)} destruct(self.csolver.get_display_struct,'csolver_') ...
                ];
            s = struct(c{:});
        end
        %------------------------------------------------------------------------------------------
        function solver = bsolver(self)
            % [Object method] Easy access to the objects' BSolver.
            solver = self.parms.p.bsolver.value;
        end
        %------------------------------------------------------------------------------------------
        function solver = csolver(self)
            % [Object method] Easy access to the objects' CSolver.
            solver = self.parms.p.csolver.value;
        end
        %--------------------------------------------------------------------------------------
    end    
    %------------------------------------------------------------------------------------------
    methods (Static)
        %--------------------------------------------------------------------------------------
        function b = De_aliasing( varargin )%bu1,uf,bu2,us)
            % [class method] recover from aliasing.
            %   explanation: see paper annie&wen-shin
            
            fa = Parameters('BSolverComposite.De_aliasing:recover from aliasing after decimation.' ...
                   , {'b1:double:subsampled b'                                            , Required ...
                     ,'b1_rate:@is_integer&@is_strictly_positive:subsampling rate of b1'  , Required ...
                     ,'b2:double:subsampled b'                                            , Required ...
                     ,'b2_shift:@is_integer&@is_strictly_positive:de-aliasing shift of b2', Required ...
                     ,'threshold:@is_strictly_positive'                                   , 1 ...
                     } ...
                   , varargin{:} ...
                   );
            b1 = fa.p.b1.value;
            b2 = fa.p.b2.value;
            u  = fa.p.b1_rate.value;
            s  = fa.p.b2_shift.value;
            threshold = fa.p.threshold.value;

            % report the position of the poles of the signal with the
            % original sampling frequency:
            b1 = power( b1, 1/u );
            b2 = power( b2, 1/s );
            
            % do not consider the damping factor, i.e. radially projcect
            % the points on the unit sphere.
            damp1 = abs(b1); % store for reset later
            b1 = b1 ./ damp1;
            b2 = b2 ./ abs(b2);

            nterms = length(b1);
            bu = zeros( nterms, u );
            bs = zeros( nterms, s );
            bu(:,1) = b1;
            bs(:,1) = b2;
            exp_ang1 = exp(2i*pi/u);
            for i = 2:u
                bu(:,i) = bu(:,i-1) .* exp_ang1;
            end
            exp_ang2 = exp(2i*pi/s);
            for i = 2:s
                bs(:,i) = bs(:,i-1) .* exp_ang2;
            end

            % Intersect ---------------------------------------

            % Cerco quelle vicine fra loro per individuare l'intersezione
            % Creo una matrice delle distanze e cerco l'elemento più piccolo, se non lo
            % trovo vuol dire che è una frequenza relativa al rumore (perchè??)

            distance_matrix = zeros(s,u);
            final_b = [];
            final_indx = 1;
            for i = 1:nterms
                % create the distance matrix every frequency 
                for j = 1:s
                    % Ciclo sulle righe della matrice distanza
                    distance_matrix(j,:) = abs(bu(i,:) - ones(1,u).*bs(i,j));
                end
                % find the minimum (treating the distance matrix as a 1d
                % array, by concatenating columns)
                [value, indx] = min(distance_matrix(:));
                col = ceil(indx/s);
                row = mod(indx,s); if row == 0; row = s; end

                % Se il minimo non è piccolo siamo in presenza di rumore
                if value > threshold
                    Sparsimatics.Warn( fa, 'Exceeding THRESHOLD: Distance = %f,  Freq1 = %f, Freq2 = %f' ...
                        , value, real(log(bu(i,col))/(2*pi*1i)), real(log(bs(i,row))/(2*pi*1i)) ...
                        );
                else
                    % calcolo tramite media dando più peso al primo...funziona meglio
                    % usare solo il primo valore
                    % final_b(final_indx) = (bu(i,col) + bs(i,row));

                    % calcolo usando solo il primo valore e ci rimetto il damping
                    final_b(final_indx) = bu(i,col)*damp1(i); %#ok<AGROW>

                    final_indx = final_indx + 1;
                end
            end

            b = transpose(final_b(:));

            k = find( isnan(b) );
            if ~isempty(k)
                Sparsimatics.Warn( self.parms.get_signature ...
                    ,[   'Some base terms (%d) have been discarded because they are NaN.' ...
                      '\n!   Discarded terms: %d.'] ...
                    , length(k), k ...
                    );
                b(k) = [];
            end
        end
        %--------------------------------------------------------------------------------------
    end
end
%--------------------------------------------------------------------------------------
% local functions
%--------------------------------------------------------------------------------------
function c = destruct(s,prefix)
    fields = fieldnames(s);
    values = struct2cell(s);
    nfields = length(fields);
    if nargin>1
        for i=1:nfields
            fields{i} = [prefix fields{i}];
        end
    end
    c = cell(2,nfields);
    c(1,:) = fields;
    c(2,:) = values;
end
%--------------------------------------------------------------------------------------