classdef BSolverEsprit < BSolverSimple
    % A class for estimating (fitting) base terms from a *Signal*, using the ESPRIT algorithm.
    %
    % Inherits BSolver.
    %
    methods
        %------------------------------------------------------------------------------------------
        function self = BSolverEsprit(varargin)
            % [constructor] Create a _b_ coefficient solver that uses the
            % ESPRIT parametric method. 
            %
            % Esprit uses a rectangular nrows x ncols system satisfying
            %
            %   * nsamples = nrows+ncols-1
            %   * nterms <= ncols
            %
            % Hence the following are sufficient argument specifications
            % 
            %   * nrows (optionally with nsamples)
            %   * ncols (optionally with nsamples)
            %   * nrows and ncols
            %
            % If nterms is not specified it is set equal to ncols.
            %
            % The nsamples parameter is always applied first. 
            % E.G. if a signal has 100 samples, and the nsamples parameter
            % is set to .9 only 90 samples are used. If, additionally, the
            % nrows parameter is set to .50, a system of 0.50x90 = 45 rows
            % is constructed. 
            %
            % `examples <tutorial_BSolver.html>`_
            %
            if ~isempty(varargin)
                self.init( varargin{:} );
            else
                % a suitable default value
                self.init('--ncols',1/3);
            end
        end
        %------------------------------------------------------------------------------------------
        function init_system_parameters( self )
            % [object method] initialize system parameters.
            % Esprit uses a rectangular Hankel matrix of size mxn and
            % m+n-1 samples.
            % nterms is independent of m and n, but is bounded by
            % nsamples/2.
            
            init_system_parameters@BSolverSimple(self);
            nsamples = self.gep.nsamples;
            nterms   = [];
            if self.parms.p.nrows.value==-1 && self.parms.p.ncols.value==-1
                % set an appropriate default value;
                ncols_value = 1/3;
%                 self.parms.p.ncols.set_validated_value(ncols_value);
                % apply it
                n = ceil(ncols_value*nsamples);
                m = nsamples+1-n;
                if self.parms.p.nterms.value~=-1
                    nterms = apply_fraction_or_min(self.parms.p.nterms.value,nsamples);
                    min_mn = min(m,n);
                    if nterms>min_mn
                        % we must adjust m and n to solve for nterms terms
                        if min_mn==n
                            %increase number of columns
                            d = nterms-n;
                            n = n+d;
                            m = m-d;
                        else
                            %increase number of rows
                            d = nterms-m;
                            m = m+d;
                            n = n-d;
                        end                            
                    end
                end
            elseif self.parms.p.nrows.value~=-1 && self.parms.p.ncols.value==-1
                % nrows only
                m = apply_fraction_or_min(self.parms.p.nrows.value,nsamples);
                n = nsamples + 1 - m;
            elseif self.parms.p.nrows.value==-1 && self.parms.p.ncols.value~=-1
                % ncols only
                n = apply_fraction_or_min(self.parms.p.ncols.value,nsamples);
                m = nsamples + 1 - n;
            else
                % nrows and ncols
                m = apply_fraction_or_min(self.parms.p.nrows.value,nsamples);
                n = apply_fraction_or_min(self.parms.p.ncols.value,nsamples);
                nsamples = m + n - 1;
                if nsamples>self.gep.nsamples
                    Sparsimatics.Error( 'BSolverEsprit.init_system_parameters' ...
                      , 'The number of samples in the signal (%d) is too small for constructing a %d x %d Hankel matrix.'...
                      , nsamples, m, n ...
                    );
                end                
            end
            
            % check validity of m, n and nsamples    
            if m+n>nsamples+1
                Sparsimatics.Error( 'BSolverEsprit.init_system_parameters' ...
                  , 'The number of samples in the signal (%d) is too small for constructing a %d x %d Hankel matrix.'...
                  , nsamples, m, n ...
                );
            end
            if m==0 || n==0
                Sparsimatics.Error( 'BSolverEsprit.init_system_parameters'...
                  , 'The specified system cannot be constructed: %d x %d.'...
                  , m, n ...
                  );
            end

            self.gep.nsamples = m+n-1;
            self.gep.nrows    = m;
            self.gep.ncols    = n;
            %   m and n are the dimensions of H (see solve method below),
            %   The linear system that is being solved is (m-1)xn.
            
            % handle nterms
            if self.parms.p.nterms.value~=-1
                if isempty(nterms) 
                    nterms = apply_fraction_or_min(self.parms.p.nterms.value,self.gep.nsamples);
                end
                if nterms>min(m,n)
                    Sparsimatics.Error( 'BSolverEsprit.init_system_parameters'...
                      , 'The specified nterms (=%d) must not exceed min(ncols(=%d),nrows(=%d).'...
                      , nterms, m, n....
                    );
                end
            else
                % solve for maximum number of terms
                nterms = ceil((2/3)*self.gep.ncols);
            end
            self.gep.nterms = nterms;
            
            Sparsimatics.Warnv(2,'! Info','BSolverEsprit.init_system_parameters'...
              , ['Solving %dx%d generalized eigenvalue problem for %d terms, using %d samples.'...
                ,'\n! Algorithm used is ESPRIT.'....
                ]...
              , self.gep.nrows-1, self.gep.ncols, self.gep.nterms, self.gep.nsamples ...
              );
            
        end
        %------------------------------------------------------------------------------------------
        function b = solve( self, varargin )
            % [object method] Estimate the normalized base parameters *b* from *signal*. 
            % This BSolver requires that the number of system rows and the number of base parameters
            % are set in advance, or can be guessed. 
            % Set *Sparsimatics.Verbosity>0* to report guesses.
            %
            % :param signal: The *Signal* object from which to estimate the base parameters.
            %
            % :return: column vector with the estimated base parameters.
            %
            % `examples <tutorial_BSolver.html>`_
            %
%             tmr =tic;
            self.parms.add_parameters( {'signal:Signal',Required}, varargin{:} );
%             fprintf('**** %g s\n',toc(tmr));

            self.init_system_parameters()

            nrows  = self.gep.nrows;
            ncols  = self.gep.ncols;
            nterms = self.gep.nterms;
            
            H = hankel( self.parms.p.signal.value.samples(1:nrows), self.parms.p.signal.value.samples(nrows:nrows+ncols-1) );
            
            % TODO change the dimension of the Hankel matrix (in order to
            % improve the autocorrelation)            
            
%             tic;
            
            [U,~,~] = svd(H,0); % S and V not needed           
%{
            if nargin < 2 || n_element == 0
                disp('Automatically chose the number of components')
                disp('TODO -- Not implemented yet..just set the maximum')
                n_element = size(U,2)-1; % I lose one dimension due to the rotation!
            end
%}
            U0 = U( 1:end-1, 1:nterms );
            U1 = U( 2:end  , 1:nterms );

            % To add LS-ESPRIT e TLS-ESPRIT
            b = eig(U0\U1); %This method works since it is computing eigenvalues of an almost diagonal matrix
            
%             t=toc; fprintf('BSolverEsprit solve (w/o init): %f\n',t);
            
            b = b.';
        end
        %------------------------------------------------------------------------------------------
    end
    
end
