classdef BSolverVexpa < BSolverComposite
    % A class for estimating (fitting) base terms from a *Signal*, using
    % the vexpa approach.  
    %
    % Inherits BSolver.
   
    %--------------------------------------------------------------------------------------
    methods
        %----------------------------------------------------------------------------------
        function self = BSolverVexpa(varargin)
            % [constructor] Create a base term solver that uses the
            % vexpa approach. Details are found in 
            %
            %   Briani, Cuyt and Lee (2017) VEXPA: Validated EXPonential
            %   Analysis through regular sub-sampling.
            %
            % This method makes use of appropriate BSolver and CSolver
            % objects to solve the sub-sampled systems.
            %
            % :param key,value=positive_integer [rate]: undersampling rate
            %   (decimation)
            % :param key,value=positive_integer [shift]: shift for
            %   unaliasing
            % :param key,value=double [M]: if M>2 solve eq (12) in
            %   vexpa.pdf, if M=2 solve by division (following
            %   subnyquist.pdf eq (10))
            % :param key,value=double [u-epsilon]: epsilon parameter for
            %   dbscan for uL-cluster analysis
            % :param key,value=double [u-minpts]: minpts parameter for
            %   dbscan for uL-cluster analysis
            % :param key,value=double [s-epsilon]: epsilon parameter for
            %   dbscan for sL-cluster analysis
            % :param key,value=double [s-minpts]: minpts parameter for
            %   dbscan for sL-cluster analysis
            self.init();
            self.parms.add_parameters ...
              ( {['rate     :@is_integer&@is_strictly_positive'...
                           ':sub-sampling rate (''u'' in the paper)'    ], 5 ...
                ,['shift   :@is_integer&@is_strictly_positive'...
                           ':shift for de-aliasing (''s'' in the paper)'], 7 ...
                ,['M        :@is_integer&@is_strictly_positive'...
                           ':number of samples for retrieving '...
                           's_lambda_i (''M'' in the paper)'           ], 8 ...
                ,['u_epsilon:@is_strictly_positive'...
                           ':DBSCAN epsilon parameter'                  ], 0.05 ...
                ,['u_minpts :@is_integer&@is_strictly_positive'...
                           ':DBSCAN minpts parameter '                  ], 5 ...
                ,['s_epsilon:@is_strictly_positive'...
                           ':DBSCAN epsilon parameter'                  ], 0.05 ...
                ,['s_minpts :@is_integer&@is_strictly_positive'...
                           ':DBSCAN minpts parameter '                  ], 5 ...
                ,['plot     :logical'...
                           ':produce graphical output'                  ], false ...
                ,['time     :logical'...
                           ':print timings'                             ], false ...
                } ...
              , varargin{:} ...
              );
            
            % further input validation:
            if ~( gcd( self.parms.p.rate.value ...
                     , self.parms.p.shift.value )==1 )
                Sparsimatics.Error( fa.get_signature ...
                    ,'The sub-sampling rate (=%d) and the de-aliasing shift (=%d) must be mutually coprime.' ...
                    , self.parms.p.rate.value ...
                    , self.parms.p.shift.value ...
                    );
            end
        end
        %------------------------------------------------------------------------------------------
        function [b,failed_sub_samples,b_uLc,uL_noise] = solve( self, varargin )
            % [object method] Estimate the normalized base parameters *b*
            % from *signal*.
            %
            % :returns: 
            %   * *b*, a column vector with the estimated base parameters
            %   * *failed_rows*, a list of sample indices that did not
            %     contribute to any of the *b* clusters. This can be used
            %     to remove the corresponding rows from the Vandermonde
            %     matrix and the right hand side for solving for the
            %     coefficients. This may give a better solution. 
            %   * *b_uLc*, cell array of the sub-sample b-clusters for each
            %     term. 
            %   * *uL_noise*, a list of indices of 
            % :param signal: The *Signal* object from which to estimate the
            %   base parameters. 
            %
%             tmr =tic;
            self.parms.add_parameters( {'signal:Signal',Required}, varargin{:} );
%             fprintf('**** %g s\n',toc(tmr));
            self.init_system_parameters()
                        
            u       = self.parms.p.rate.value;
            s       = self.parms.p.shift.value;
            M       = self.parms.p.M.value;
            u_epsilon = self.parms.p.u_epsilon.value;
            u_minpts  = self.parms.p.u_minpts.value;
            s_epsilon = self.parms.p.s_epsilon.value;
            s_minpts  = self.parms.p.s_minpts.value;
            plotres = self.parms.p.plot.value;
            signal  = self.parms.p.signal.value;
            tictoc  = self.parms.p.time.value;
            
            bs = self.bsolver;
            cs = self.csolver;
            % create a bsolver of the same class as self.bsolver that
            % solves for a single term
            se = ['bs1 = ' class(bs) '(''--nterms'',1);']; 
            eval(se);
            
            sub_m = cell(1,M);
            alpha_m = [];
            
            if tictoc
                dt_bs = 0;
                dt_cs = 0;
                t0 = tic;
            end
            for k=0:u-1
                % construct M subsampled signals with subsampling rate u and
                % shifts k+m*s and store them in sub_m :
                for m=0:(M-1)
                    % m=0 : is for lambda_u and uL 
                    % m=1,... : m * de-aliasing shift
                    sub_m{m+1} = signal.select(u,'--shift', k + m*s );
                    %plot_signal(sub_m{m+1});
                end
                
                if tictoc; tk = tic; end
                  lambda_u = bs.solve( sub_m{1} );
                if tictoc; dt_bs = dt_bs+toc(tk); end
                
            %     plot_base_terms(lambda_u)
                if isempty( alpha_m )
                    % this is true when k==0, i.e. the first iteration
                    nb = length(lambda_u);
                    bs.init('--nterms',nb);
                    alpha_m = zeros(M,nb);
                    uL = zeros(u,nb);
                    sL = zeros(u,nb);
                end
                for m=0:(M-1)
                    if tictoc; tk = tic; end
                      alpha_m(m+1,:) = cs.solve( sub_m{m+1}, lambda_u ); %#ok<AGROW>
                    if tictoc; dt_cs = dt_bs+toc(tk); end
                end
                if M>2
                    lambda_s = zeros(1,nb);
                    for i=1:nb
                        s_alpha_m_i = Signal(1,alpha_m(:,i));
                        if tictoc; tk = tic; end
                          lambda_s(i) = bs1.solve(s_alpha_m_i);
                        if tictoc; dt_bs = dt_bs+toc(tk); end
                    end
                elseif M==2
                    lambda_s = alpha_m(2,:)./alpha_m(1,:);
                else
                    Sparsimatics.Error('BSolverVexpa.solve()'...
                        ,'parameter M must be >= 2.' ...
                        );
                end
                uL(k+1,:) = lambda_u; 
                sL(k+1,:) = lambda_s; 
            end
            if tictoc
                dt_decimation = toc(t0);
            end

            % perform cluster analysis on uL
            if tictoc
                tic;
            end
            [ ~, uL_cl, uL_rw] = find_clusters( uL, '--epsilon', u_epsilon ...
                                                  , '--minpts' , u_minpts  ...
                                                  , '--plot'   , plotres ...
                                              );
            if tictoc
                dt_cluster_uL = toc;
            end
            
            uL_id = (1:numel(uL_cl))';
            sL = sL(:);
            if plotres
                title('uL') % for the plot produced by find_clusters above.
                plot_clusters( sL, uL_cl, uL_rw, '--plot-noise', false );
                title('sL')
            end
            
            nclusters = max(uL_cl);
            bu = zeros(1,nclusters);
            bs = zeros(1,nclusters);
            if nargout>=2
                failed_rows = [];
            end
            b_uLc = cell(size(bu)); % the sub-sample clusters for each b
            if tictoc
                tic;
            end
            for c=1:nclusters
                % take the sL points for each cluster and carry out a 
                % cluster analysis on them:
                uL_cl_EQ_c = (uL_cl==c);
                sLc =    sL(uL_cl_EQ_c);
                ids = uL_id(uL_cl_EQ_c); % the indices of the points in this uL cluster
                [~,cl,~] = find_clusters( sLc, '--epsilon', s_epsilon ...
                                             , '--minpts' , s_minpts  ...
                                             , '--plot'   , plotres ...
                                        );
                if plotres
                    title(sprintf('sL cluster%d',c))
                end
                nclusters2 = max(cl);

                if nclusters2==0
                    % the sL points corresponding to cluster c do NOT
                    % cluster, hence they are noise points that
                    % accidentally ended up in a cluster, usually the
                    % number of points is smaller than u.
                    bu(c) = nan;
                    bs(c) = nan;
                else
                    if nclusters2==1
                        % There is one cluster, keep only the points in the
                        % cluster
                        subset = (cl==1);
                        sLc = sLc(subset);
                        ids = ids(subset);
                        % these indices made it into a cluster, and thus
                        % contribute to a true b-term.
                        uL_id(ids) = 0;
                        % In the end the nonzero entries in uL_id all
                        % correspond to noise terms
                    else
                        % There is more than one cluster: Unsure what this
                        % actually means. Maybe it falls apart because of
                        % an unlucky choice of the DBSCAN epsilon
                        % parameter. 
                        tbl = tabulate(cl);
                        % For now we go with the largest cluster
                        % find the cluster with the most elements
                        
                        [~,I] = sort(tbl(:,2),'descend');
                        largest = tbl(I(1),1);
                        subset = (cl==largest);
                        sLc = sLc(subset);
                        ids = ids(subset);
                        % these indices made it into a cluster, and thus
                        % contribute to a true b-term.
                        uL_id(ids) = 0;
                        % In the end the nonzero entries in uL_id all
                        % correspond to noise terms
                        
                        if Sparsimatics.Verbosity>0
                            % Spit out some information on this case:
                            msg = [];
                            for i=1:size(tbl,1)
                                if tbl(i,1)==0
                                    msg = [msg '\n!   Noise   : ' num2str(tbl(i,2)) ' points']; %#ok<AGROW>
                                else
                                    msg = [msg '\n!   cluster' num2str(tbl(i,1)) ': ' num2str(tbl(i,2)) ' points']; %#ok<AGROW>
                                end
                            end
                            Sparsimatics.WARN(1,'WARNING','BSolverVexpa.solve' ...
                              , ' For uL cluster%d the sL values form %d clusters:' ...
                              , msg ...
                              ,['\n! Continuing analysis with cluster' num2str(I(1)) '.'] ...
                            );
                        end
                    end
                    subset = (uL_cl==c);
                    uLc = uL(subset);
                    rwc = uL_rw(subset);
                    if nargout>=2
                        % remove rows which are not in the cluster:
                        % which rows did not contribute to the cluster:
                        non_contributing_rows = rwc(cl==0);
                        if ~isempty(non_contributing_rows)
                            % there are non-contributing rows
                            for ir = 1:numel(non_contributing_rows)
                                if isempty(find(failed_rows==non_contributing_rows(ir),1))
                                    % this non-contributing row has not
                                    % been added before
                                    failed_rows = [failed_rows non_contributing_rows(ir)]; %#ok<AGROW>
                                end
                            end
                        end
                    end
                    b_uLc{c} = uLc;
                    bu(c) = average(uLc);
                    bs(c) = average(sLc);
                end
                uL_noise = uL(uL_id~=0);
            end
            if tictoc
                dt_cluster_sL = toc;
            end
            
            % remove the NaN entries
            isanumber = ~isnan(bs);
            bu = bu(isanumber);
            b_uLc = b_uLc(isanumber);
            bs = bs(isanumber);
            
            if tictoc
                tic;
            end
            b = BSolverVexpa.De_aliasing(bu,u,bs,s);
            if tictoc
                dt_de_alias = toc;
            end
            if nargout>=2
                rmv = [];
                for i=1:length(failed_rows)
                    rmv = [rmv failed_rows(i):u:signal.length];   %#ok<AGROW>
                end
                failed_sub_samples = rmv;
            end
            
            if tictoc
                dt_total = toc(t0);
                fprintf('BSolverVexpa   Decimation phase   : %f (%6.2f%%)\n'              ,dt_decimation,100*dt_decimation/dt_total);
                fprintf('BSolverVexpa   cluster analysis uL: %f (%6.2f%%)\n'              ,dt_cluster_uL,100*dt_cluster_uL/dt_total);
                fprintf('BSolverVexpa   cluster analysis sL: %f (%6.2f%%) (%d clusters)\n',dt_cluster_sL,100*dt_cluster_sL/dt_total,nclusters);
                fprintf('BSolverVexpa   de-aliasing        : %f (%6.2f%%)\n'              ,dt_de_alias  ,100*dt_de_alias  /dt_total);
                dt_sum = dt_decimation+dt_cluster_uL+dt_cluster_sL+dt_de_alias;
                fprintf('BSolverVexpa   sum                : %f (%6.2f%%)\n'              ,dt_sum       ,100*dt_sum       /dt_total);
                fprintf('BSolverVexpa   total (w/o init)   : %f (%6.2f%%)\n'              ,dt_total     ,100*dt_total     /dt_total);
                fprintf('BSolverVexpa   bsolvers           : %f (%6.2f%%)\n'              ,dt_bs        ,100*dt_bs        /dt_total);
                fprintf('BSolverVexpa   csolvers           : %f (%6.2f%%)\n'              ,dt_cs        ,100*dt_bs        /dt_total);
            end
        end 
        %------------------------------------------------------------------------------------------
        function s = get_display_struct(self)
            s = struct;
            has_signal = true;
            try 
                self.parms.p.signal
            catch
                has_signal = false;
            end
            bsolver_display_struct = self.bsolver.get_display_struct;
            csolver_display_struct = self.csolver.get_display_struct;
            if has_signal
                s.signal       = self.parms.p.signal.value.label;
%                 s.nsamples     = [ num2str(self.parms.p.nsamples.value) ' -> ' num2str(self.gep.nsamples ) ];
%             else
%                 s.nsamples     = self.parms.p.nsamples.value;
            end
            s.bsolver          = class(self.bsolver);
            if has_signal
              s.bsolver_signal = bsolver_display_struct.signal;
            end
            s.bsolver_nsamples = bsolver_display_struct.nsamples;
            s.bsolver_nrows    = bsolver_display_struct.nrows;
            s.bsolver_ncols    = bsolver_display_struct.ncols;
            s.bsolver_nterms   = bsolver_display_struct.nterms;
            s.csolver          = class(self.csolver);
            if has_signal
              s.csolver_signal = csolver_display_struct.signal;
            end
            s.csolver_nsamples = csolver_display_struct.nsamples;
            s.csolver_nrows    = csolver_display_struct.nrows;
            s.csolver_ncols    = csolver_display_struct.ncols;
            s.csolver_delta    = csolver_display_struct.delta;
        end
        %----------------------------------------------------------------------------------
    end
    %--------------------------------------------------------------------------------------
    %------------------------------------------------------------------------------------------
    methods (Static)
        %--------------------------------------------------------------------------------------
        function b = De_aliasing( varargin )%bu1,uf,bu2,us)
            % [class method] recover from aliasing.
            %   explanation: see paper annie&wen-shin
            
            fa = Parameters('BSolverSubsampling.De_aliasing:recover from aliasing after decimation.' ...
                   , {'b1:double:subsampled b'                                            , Required ...
                     ,'b1_rate:@is_integer&@is_strictly_positive:subsampling rate of b1'  , Required ...
                     ,'b2:double:subsampled b'                                            , Required ...
                     ,'b2_shift:@is_integer&@is_strictly_positive:de-aliasing shift of b2', Required ...
                     ,'threshold:@is_strictly_positive'                                   , 1 ...
                     } ...
                   , varargin{:} ...
                   );
            b1 = fa.p.b1.value;
            b2 = fa.p.b2.value;
            u  = fa.p.b1_rate.value;
            s  = fa.p.b2_shift.value;
            threshold = fa.p.threshold.value;

            % report the position of the poles of the signal with the
            % original sampling frequency:
            b1 = power( b1, 1/u );
            b2 = power( b2, 1/s );
            
            % do not consider the damping factor, i.e. radially projcect
            % the points on the unit sphere.
            damp1 = abs(b1); % store for reset later
            b1 = b1 ./ damp1;
            b2 = b2 ./ abs(b2);

            nterms = length(b1);
            bu = zeros( nterms, u );
            bs = zeros( nterms, s );
            bu(:,1) = b1;
            bs(:,1) = b2;
            exp_ang1 = exp(2i*pi/u);
            for i = 2:u
                bu(:,i) = bu(:,i-1) .* exp_ang1;
            end
            exp_ang2 = exp(2i*pi/s);
            for i = 2:s
                bs(:,i) = bs(:,i-1) .* exp_ang2;
            end

            % Intersect ---------------------------------------

            % Cerco quelle vicine fra loro per individuare l'intersezione
            % Creo una matrice delle distanze e cerco l'elemento più piccolo, se non lo
            % trovo vuol dire che è una frequenza relativa al rumore (perchè??)

            distance_matrix = zeros(s,u);
            final_b = [];
            final_indx = 1;
            for i = 1:nterms
                % create the distance matrix every frequency 
                for j = 1:s
                    % Ciclo sulle righe della matrice distanza
                    distance_matrix(j,:) = abs(bu(i,:) - ones(1,u).*bs(i,j));
                end
                % find the minimum (treating the distance matrix as a 1d
                % array, by concatenating columns)
                [value, indx] = min(distance_matrix(:));
                col = ceil(indx/s);
                row = mod(indx,s); if row == 0; row = s; end

                % Se il minimo non è piccolo siamo in presenza di rumore
                if value > threshold
                    Sparsimatics.Warn( fa, 'Exceeding THRESHOLD: Distance = %f,  Freq1 = %f, Freq2 = %f' ...
                        , value, real(log(bu(i,col))/(2*pi*1i)), real(log(bs(i,row))/(2*pi*1i)) ...
                        );
                else
                    % calcolo tramite media dando più peso al primo...funziona meglio
                    % usare solo il primo valore
                    % final_b(final_indx) = (bu(i,col) + bs(i,row));

                    % calcolo usando solo il primo valore e ci rimetto il damping
                    final_b(final_indx) = bu(i,col)*damp1(i); %#ok<AGROW>

                    final_indx = final_indx + 1;
                end
            end

            b = transpose(final_b(:));

            k = find( isnan(b) );
            if ~isempty(k)
                Sparsimatics.Warn( self.parms.get_signature ...
                    ,[   'Some base terms (%d) have been discarded because they are NaN.' ...
                      '\n!   Discarded terms: %d.'] ...
                    , length(k), k ...
                    );
                b(k) = [];
            end
        end
        %--------------------------------------------------------------------------------------
    end
end

