classdef CSolver < handle & matlab.mixin.CustomDisplay
    % Abstract class for *c* coefficient solvers. Derived classe must implement a::
    %
    %   c = CSolverSubclass.solve( self, signal )
    %
    % method that computes the *c* coefficients from a *Signal* object . 
    %
    properties
        parms    % Parameters object with solver parameters
    end
    properties(SetAccess=protected,GetAccess=public)
        % properties of the 
        vdm = struct('nrows'   , [] ...
                    ,'ncols'   , [] ...
                    ,'nsamples', [] ...
                    ,'delta'  , [] ...
                    );
    end
    

    methods 
        %------------------------------------------------------------------------------------------
        function self = init( self, varargin )
            % [delayed constructor] Initialize (or re-initialize) a
            % BSolver. This sets the following property
            %
            %   * self.parms.p.nsamples, and
            %
            % The init method checks that these properties are in the set
            % { -1, x:0<x<1, 1, 2, 3, ...}. The validity of the parameter
            % combination is to be validated further by the derived class,
            % usually in the init_system_parameters (abstract) method,
            % which is normally called at the beginning of the solve
            % (abstract) method. 
            %
            % :param key,value=double nrows: the number of rows that the
            %   solver will use (integer or in ]0,1[) - if configurable. 
            % :param key,value=double delta: the rows correspond to
            %   sample 1, 1+delta, 1+2*delta, ... The default is -1 and
            %   sets delta = floor(signal.length/nrows)
            %
            % :return: self.
            %
            self.parms = Parameters( class(self) ...
                          , { 'nrows   :@is_minus1_or_fraction_or_positive_integer', -1 ...
                            , 'ncols   :@is_minus1_or_fraction_or_positive_integer', -1 ...
                            , 'nsamples:@is_minus1_or_fraction_or_positive_integer', -1 ...
                            , 'delta   :@is_integer: use every delta-th sample for the system rows.', -1 ...
                            } ...
                          , varargin{:} ...
                          );
        end
        %------------------------------------------------------------------------------------------
        init_system_parameters(self)
            % [abstract method] Initialize solver parameters. 
        %------------------------------------------------------------------------------------------
        c = solve( self, signal, b ) 
            % [abstract method] Estimate the *c* coefficients from *signal*
            % for given base parameters *b*. 
        %------------------------------------------------------------------------------------------
        function cp = copy(self)
            % [] Perform a deep copy of a CSolver.
            % See BSolver.copy for an explanation of the issues involved.
            
            % first make new (derived) CSolver object
            cp = feval(class(self));
            % make a shallow copy of the parms property
            new_parms = copy(self.parms);
            % and overwrite cp.parms
            cp.parms = new_parms;
        end
        %--------------------------------------------------------------------------------------
        function s = get_display_struct(self)
            s = struct;
            try 
                self.parms.p.signal
                s.signal   = self.parms.p.signal.value.label;
                s.nsamples = [ num2str(self.parms.p.nsamples.value) ' -> ' num2str(self.vdm.nsamples) ];
                s.nrows    = [ num2str(self.parms.p.nrows.value)    ' -> ' num2str(self.vdm.nrows   ) ];
                s.ncols    = [ num2str(self.parms.p.ncols.value)    ' -> ' num2str(self.vdm.ncols   ) ];
                s.delta    = [ num2str(self.parms.p.delta.value)    ' -> ' num2str(self.vdm.delta   ) ];
            catch
                s.nsamples = self.parms.p.nsamples.value;
                s.nrows    = self.parms.p.nrows.value;
                s.ncols    = self.parms.p.ncols.value;
                s.delta    = self.parms.p.delta.value;
            end
        end
        %--------------------------------------------------------------------------------------
    end
    %------------------------------------------------------------------------------------------
    methods (Access=protected)
        %------------------------------------------------------------------------------------------
        function propgrp = getPropertyGroups(self)
            if ~isscalar(self)
                propgrp = getPropertyGroups@matlab.mixin.CustomDisplay(self);
            else
                propgrp = matlab.mixin.util.PropertyGroup(self.get_display_struct);
            end
        end
        %------------------------------------------------------------------------------------------
    end
    %------------------------------------------------------------------------------------------
end

