function [xy,cluster_index,row_index] = find_clusters(varargin)
    % [function] Perform a cluster analysis of points in 2D using DBSCAN.
    %
    % :param points: nxm matrix of points represented by a complex number.
    %   If n>1 it is assumed that the n rows represent the outcome of n
    %   replicas of the same experiment. E.g. in a VEXPA analysis there u
    %   sets of b-values, there n=u. As the cluster analysis is expected to
    %   cluster points from different rows and not from the same row,
    %   points from different rows are drawn with different colors. (DBSCAN
    %   does, however, not necessarily respect this expectation by itself).
    % :param key,value=logical [plot]: produce a plot of the results.
    %   Default is true. 
    % :param key,value=double epsilon: DBSCAN epsilon parameter. Default is
    %   5e-2. 
    % :param key,value=positive_int [minpts]: DBSCAN minpts parameter.
    %   Default is 3. 
    % :returns: *xy* is a n*m x 2 array of points *xy(i,1)* is the real part of
    %   *points*, *xy(i,2)* the imaginary part. *cluster_index* is a n*m x
    %   1 array with the number of the cluster to which the point belongs
    %   (a 0 identifies a point that does not belong to any cluster
    %   (noise). *row_index* is a n*m x 1 array with the row index from
    %   which the point comes.
    %
    % The parameters epsilon and minpts can be also be arrays of multiple
    % values. In that case a multi-step procedure is ran in which first
    % clusters are formed with parameters epsilon(1) and minpts(1), then
    % the remaining points are subjected to clustering with parameters
    % epsilon(2) and minpts(2), and so on.
    %
    fa = Parameters( 'find_clusters:Find clusters in 2 dimensions using DBSCAN.' ...
           , {'points :double : array of points'       , Required ...
             ,'plot   :logical: produce plot, or not'  , true ...
             ,'epsilon:@is_strictly_positive            : DBSCAN epsilon parameter', 5e-2 ...
             ,'minpts :@is_strictly_positive&@is_integer: DBSCAN minpts parameter' , 3    ...
             } ...
           , varargin{:} ...
          );
    must_plot_clusters = fa.p.plot.value;
    % create a linear array of points
    size_points = size(fa.p.points.value);
    nrows = size_points(1);
    
    % create a row_index index for points
    row_index = zeros(size_points);
    for r=1:nrows
        row_index(r,:) = r;
    end
    row_index = row_index(:);
    
    xy = zeros(numel(fa.p.points.value),2);
    xy(:,1) = real(fa.p.points.value(:));
    xy(:,2) = imag(fa.p.points.value(:));
    
    epsilon = fa.p.epsilon.value;
    minpts  = fa.p.minpts.value;
    
    % if either epsilon or minpts is non-scalar, convert the scalar into a
    % constant array of the same length:
    if numel(epsilon)>1 && numel(minpts)==1
        minpts = minpts*ones(size(epsilon));
    end
    if numel(epsilon)==1 && numel(minpts)>1
        epsilon = epsilon*ones(size(minpts));
    end
    % assert that epsilon and minpts have the same length:
    if numel(epsilon)~=numel(minpts)
          Sparsimatics.Error('find_clusters(...)' ...
          , 'epsilon and minpts must be scalar or of equal length.' ...
          );
    end
    
    n = numel(epsilon); % the number of clustering phases
    
    cluster_index = DBSCANm( xy, epsilon(1), minpts(1), row_index );
    if n>1
        n_clusters =  max(cluster_index);
        for i=2:n
            unclustered_points = find(cluster_index==0);
            if ~isempty(unclustered_points)
                unclustered_xy = xy(unclustered_points,:);
                unclustered_rw = row_index(unclustered_points);
                cl_i = DBSCANm( unclustered_xy, epsilon(i), minpts(i), unclustered_rw );
                cl = cl_i + n_clusters*(cl_i~=0);
                n_clusters = n_clusters + max(cl_i);
                cluster_index(unclustered_points) = cl;
            else
                break
            end
        end
    end
    
    if must_plot_clusters
        if nrows==1
            plot_clusters(xy, cluster_index );
        else
            plot_clusters(xy, cluster_index, row_index );
        end
    end
end
