function figure_handle = plot_real_base_terms( varargin )
    % [function] Plot only the purely real base parameters *b* in the
    % $(x,z)$-plane. The z-value is selected in the same way as for
    % :func:`plot3_base_terms`.
    %
    % :param MultiExponentialParameters p: one or more *MultiExponentialParameters* objects.
    % :param key,value=char [z]: selects the value to be plotted on the z-axis: 'abs': magnitude of *c* coefficient (default), 'angle': angle of *c* coefficient, 'energy': energy of the components. 
    % :param key,value=double [width]: if nonzero, sets the plot axis limits to ``axis( [-width,width,-width,width] );``, otherwise no axis limits are applied (showing all points plotted). 
    % :param key,value=double [duration]: when ``'--z'=='energy'`` the energy will be computed for a signal with a duration of this many seconds, default = 1s. 
    % :param key,value,double [axis]: array of radii to draw, default = [1], which just draws the unit circle.
    % :return: a handle to the created figure.
    % 
    % `Examples <tutorial_Visualization.html>`_
    %
    fa = Parameters( 'plot_real_base_terms:Plot only real base terms in the (x,z)-plane, z = abs(c), angle(c), or the component energy.' ...
           , {'ps:MultiExponentialParameters: base terms and coefficients:+', Required ...
             ,'z:char: value to be plotted on the vertical axis, ''abs''|''angle''|''energy''|''ill-disposedness''', 'abs' ...
             ,'nlargest:double: only the nlargest most energetic components will be plotted (default=0=all)',0 ...
             ,'duration:double: duration of the signal used to compute the energy when ''--z''==''energy''', 1 ...   
             ,'axis:double: [x0,x1 <,z0,z1>]=plotted range on horizontal axis, and optionally z-axis',[-1.1,1.1] ...
             } ...
           , varargin{:} ...
           );
    ps = fa.p.ps.value;
    z_value = fa.p.z.value;
    n_largest = fa.p.nlargest.value;
    duration = fa.p.duration.value;
    axis_ranges = fa.p.axis.value;
    
    figure_handle = figure('Tag' , 'plot_real_base_terms' ...
                          ,'Name', 'plot_real_base_terms' ...
                          );
    nps = length(ps);
    symbols = repmat('o',nps);
    if nps>1
        symbols(1) = '+';
    end
    
    z = cell(1,nps);
    if strcmp(z_value,'abs')
        for i=1:nps
            z{i} = abs(ps{i}.c);
        end
        zlabel_ = 'abs(c)';
    elseif strcmp(z_value,'angle')
        for i=1:nps
            z{i} = angle(ps{i}.c);
        end
        zlabel_ = 'angle(c)';
    elseif strcmp(z_value,'energy')
        for i=1:nps
            z{i} = log10(ps{i}.energies(duration*ps{i}.sampling_frequency));
        end
        zlabel_ = 'log_1_0(energy)';
    elseif strcmp(z_value,'ill-disposedness')
        for i=1:nps
            z{i} = log10( ps{i}.ill_disposedness );
        end
        zlabel_ = 'log_1_0(ill-disposedness)';
    else
        Sparsimatics.Error(fa,'Invalid z value, expecting ''abs''|''ang''|''energy'', got ''%''.',z_value);
    end    
    colors = zeros(nps,3);
    legend_ = cell(1,nps);
    if n_largest>0
        I = cell(1,nps);
    end
    for i=1:nps
        legend_{i} = num2str(i);
        
        [S,I] = sort(abs(imag(ps{i}.b)));
        % sort based on the absolute value of the imaginary part of the b
        % terms. Typically, S starts with a number of zeros, corresponding to
        % the real base terms, followed by pairs of equal values, corresponding
        % to complex conjugate pairs of base terms, or by singlet,
        % corresponding to complex base terms without complex conjugate.
        J = find(S>0); % index array with the non-zero elements of S, 
                       % this identifies the complex base terms 
                       % terms (complex conjugate pairs or singlets.
        
        if ~isempty(J) 
            if J(1)==1 % all components have nonzero imaginary part
                Sparsimatics.Warn('plot_real_base_terms','no purely real components - nothing to plot!')
                continue
            end
            J = J(1)-1;
        else
            % all components have zero imaginary part, that is there are
            % only purely real components
            J = length(ps{i}.b) ;
        end
        I = I(1:J); % indices of pure real b values
        xi = ps{i}.b(I);
        zi = z{i}(I);
        if n_largest>0
            if strcmp(z_value,'energy')
                [~,Ii] = sort(zi,'descend');
            else
                Ei = ps{i}.energies(duration*ps{i}.sampling_frequency);
                [~,Ii] = sort(Ei,'descend');
            end
            Ii = Ii(1:n_largest);
            I{i} = Ii;
            xi = xi(Ii);
            zi = zi(Ii);
        end
        if i==1
            line = stem( xi,zi,symbols(i),'Color',[1,0,0] );
            fprintf('Real base terms (filtered):\n')
        else
            line = stem( xi,zi,symbols(i) );
        end
        messagex = sprintf('  b(%d) = [%d',i,xi(1));
        messagez = sprintf('  z(%d) = [%d',i,zi(1));
        for j=2:length(xi)
            messagex = [messagex ', ' num2str(xi(j))];
            messagez = [messagez ', ' num2str(zi(j))];
        end
        messagex = [messagex ']\n'];
        messagez = [messagez ']\n'];
        fprintf(messagex)
        fprintf(messagez)

        colors(i,:) = get(line,'Color');
        hold on
    end
    
    if nps>1
        legend(legend_{:});
    end
    zlabel(zlabel_);
    
    if length(axis_ranges)>=4
        axis(axis_ranges(1:4));
    else
        axis([axis_ranges(1),axis_ranges(2),0,1]);
        axis('auto y');
    end
%     % improve daspect
%     xl = xlim;
%     dx = xl(2)-xl(1);
%     zl = zlim;
%     dz = zl(2)-zl(1);
%     daspect([1,1,dz/dx]);
end
