function figure_handle = plot_signal( varargin )
    % [function] This function creates a plot with 2 subplots, one for the real part and one for the imaginary part of *signal*. 
    % For each component the following items can be plotted: 
    %
    %   * *signal.samples*,
    %   * *signal.noise*, and 
    %   * *signal.samples* - *signal.noise*
    %
    % :param signal: the *Signal* object to be plotted.
    % :param key,value=char [plot_what]: a string containing the characters
    %   'R' (=plot real part), 'I' (=plot imaginary part), 'M' (=plot
    %   magnitude), 's' (= plot samples), 'n' (=plot noise), 'd' (=plot
    %   signal without noise). If the *signal* doesnot contain noise, 'n'
    %   and 'd' are ignored. An empty string or '*' causes everything to be
    %   plotted.      
    % :param key,value=char [time_scale]: a string identifying the time
    %   scale: 'n[ormalized]' or 'p[hysical]'. The default corresponds to
    %   the value of *Sparsimatics.Use_physical_time*.
    % :param key,value=@is_integer [range]: range of the signal to be
    %   plotted. E.g 1:50, 10:2:20. Default is all samples in the signal.
    % :return: handle of the created figure.
    %
    % `examples <tutorial_Visualization.html>`_
    %
    fa = Parameters('plot_signal:Plot a signal''s samples and noise (real part/imaginary part/magnitude) in the time domain.' ...
           , {'signal    :Signal: signal to plot'                 , Required  ...
             ,'plot_what :char  : ''RIMsnd'' '                    , '*'       ...
             ,'time_scale:char  : ''p[hysical]''|''n[ormalized]''','physical' ...
             ,'range:@is_integer : range to be plotted'           , -1        ...
             ,'no_legend:logical : plot with/out legend'          ,false      ...
             } ...
           , varargin{:} ...
           );
    signal     = fa.p.signal.value;
    plot_what  = fa.p.plot_what.value;
    time_scale = fa.p.time_scale.value;
    
    n = length(time_scale);
    if ~( strncmpi( time_scale, 'normalized', n ) ) ...
    && ~( strncmpi( time_scale, 'physical'  , n ) )
          Sparsimatics.Error( fa, 'function argument ''time_scale'' must be ''p[hysical]'' or ''n[ormalized]''!');
    end 
    figure_handle = figure( 'Tag' , signal.label ...
                          , 'Name', ['plot_signal( ',signal.label, ' )'] );

    if isempty(plot_what)
        plot_what = '*';
    end
    plot_all = contains(plot_what,'*');
    if plot_all
        plot_re= true;
        plot_im= true;
        plot_mg= true;
        plot_s = true;
        plot_n = true;
        plot_d = true;
    else
        plot_re= contains(plot_what,'R');
        plot_im= contains(plot_what,'I');
        plot_mg= contains(plot_what,'M');
        if ~plot_re && ~plot_im && ~plot_mg % if nothing was specified, by default all are plotted
            plot_re = true;
            plot_im = true;
            plot_mg = true;
        end
        plot_s = contains(plot_what,'s');
        plot_n = contains(plot_what,'n');
        plot_d = contains(plot_what,'d');
        if ~plot_s && ~plot_n && ~plot_d % if nothing was specified, by default all are plotted
            plot_s = true;
            plot_n = true;
            plot_d = true;
        end
    end
    if plot_n && isempty(signal.noise)
        plot_n = false;
    end
    if plot_d && isempty(signal.noise)
        plot_d = false;
    end
    
    legend_ = {};
    if ~fa.p.no_legend.value
        i=1;
        if plot_s
            legend_{i} = signal.label;
            i=i+1;
        end
        if plot_n
            legend_{i} = 'noise';
            i=i+1;
        end
        if plot_d
            legend_{i} = [signal.label ' without noise'];
        end
    else 
        
    end
    xlabel_ = 'time [s]';
    if time_scale(1)=='n'
        xlabel_ = 'time [1]';
    end
    
    t = signal.times(time_scale);
    if fa.p.range.value==-1
        samples = signal.samples;
        noise = signal.noise;
    else
        range_ = fa.p.range.value;
        samples = signal.samples(range_);
        noise = signal.noise(range_);
        t = t(range_);
    end

    
    n_subplots = double( int16(plot_re)+int16(plot_im)+int16(plot_mg) );
    i_subplot = 0;
    
    if plot_re
        i_subplot = i_subplot+1;
        subplot(n_subplots,1,i_subplot)
        hold_on = false;
        if plot_s
            plot( t, real( samples ) )
            hold_on = true;
        end
        if plot_n
            if hold_on
                hold on
            end
            plot( t, real( noise ),                 'r:')
            hold_on = true;
        end
        if plot_d
            if hold_on
                hold on
            end
            plot( t, real( samples - noise ), 'k')
        end
        
        if ~fa.p.no_legend.value
            legend( legend_ )
        end
        xlabel(xlabel_)
        ylabel('Re')
        title(signal.label)
    end
    
    if plot_im
        i_subplot = i_subplot+1;
        subplot(n_subplots,1,i_subplot)
        hold_on = false;
        if plot_s
            plot( t, imag( samples ) )
            hold_on = true;
        end
        if plot_n
            if hold_on
                hold on
            end
            plot( t, imag( noise ),                 'r:')
            hold_on = true;
        end
        if plot_d
            if hold_on
                hold on
            end
            plot( t, imag( samples - noise ), 'k')
        end
        
        if ~fa.p.no_legend.value
            legend( legend_ )
        end
        xlabel(xlabel_)
        ylabel('Im')
        title(signal.label)
    end
    
    if plot_mg
        i_subplot = i_subplot+1;
        subplot(n_subplots,1,i_subplot)
        hold_on = false;
        if plot_s
            plot( t, abs( samples ) )
            hold_on = true;
        end
        if plot_n
            if hold_on
                hold on
            end
            plot( t, abs( noise ),                 'r:')
            hold_on = true;
        end
        if plot_d
            if hold_on
                hold on
            end
            plot( t, abs( samples - noise ), 'k')
        end
        
        if ~fa.p.no_legend.value
            legend( legend_ )
        end
        xlabel(xlabel_)
        ylabel('magnitude')
        title(signal.label)
    end
end
