function plot_unit_circle(varargin)
    % [function] Utility function that adds the complex unit circle to the current plot.
    %
    % :param key,value,double [width]: if nonzero, sets the plot axis limits as ``axis( [-width,width,-width,width] );``, otherwise width is set to 2.
    % :param key,value,double [dim]: number of dimension of the plot: 2|3.
    % :param key,value,double [radii]: array of radii to draw, default = [1].
    %
    % `examples <tutorial_Visualization.html>`_
    %
    fa = Parameters ...
           ( 'plot_unit_circle:Utility function to draw the unit circle on top of the current figure.' ...
           , {'width:double: axis limits'           , 2 ...
             ,'dim  :double: plot dimensions 2|3'   , 2 ...   
             ,'radii:double: array of radii to draw', 1 ...
             } ...
           , varargin{:} ...
          );
    w     = fa.p.width.value;
    dim   = fa.p.dim.value;
    radii = fa.p.radii.value;

    % turn legend AutoUpdate off so the axes and the unit circle do not
    % show up in the legend. It is turned on again before returning.
    current_legend_handle = findobj(gcf,'Tag','legend');
    if ~isempty(current_legend_handle)
        current_legend_handle.AutoUpdate = 'off';
    end
    
    hold on;
    daspect([1,1,1]);
    persistent x_iy n
    if isempty(x_iy)
        n=1000;
        x_iy = exp((2*pi/n)*1i*(0:n));
    end
    nr = length(radii);
    if dim == 2
        for ir=1:nr
            plot(radii(ir)*x_iy,'k:'); % circle
        end
        plot([-2,2], [0,0], 'k-');% x-axis
        plot([0,0], [-2,2], 'k-');% y-axis
    elseif dim == 3
        for ir=1:nr
            plot3( radii(ir)*real(x_iy), radii(ir)*imag(x_iy), zeros(1,n+1),'k:');
        end
        plot3([-2,2], [0,0], [0,0], 'k-');% x-axis
        plot3([0,0], [-2,2], [0,0], 'k-');% y-axis
    else
        Sparsimatics.Error(fa,'Number of plot dimensions must be 2|3, got %s.',dim);
    end
        
    if w~=0
        axis( [-w,w,-w,w] );
    end
    
    xlabel('Re(b)');
    ylabel('Im(b)');

    if ~isempty(current_legend_handle)
        current_legend_handle.AutoUpdate = 'off';
    end
  
    hold off;
end